// Hartley.HPP
// Author  : Mike Nice 73565,565

#ifndef _HARTLEY_HPP
#define _HARTLEY_HPP


/**********************************************************************/
/*                 PROCEDURE  FHT                                     */
/**********************************************************************/
/*
  The FHT subroutine uses a Hartley transform to calculate the Fourier
  transform in one half the time of the standard Cooley-Tukey FFT 
  algorithm.

  HARTLEY TRANSFORM:

              N-1
              ___
              \
    H(k)  =   /     X(i) * [cos(2*PI*i*k/N) + sin(2*PI*i*k/N)]
              ---
              i=0

 IN: function - calcFHT: calculate fourier coefficients
             inverseFHT: recreate waveform
     numberOfDataPoints: # of data points in arrays,
                         MUST BE A POWER OF 2.
     inputTable: can be-
         o Data points to calculate fourier coefficients from (calcFHT)
           inputTable will be unmodified.
         o fourier coefficients to use in recreating waveform (inverseFHT)

OUT: resultTable: can be-
        o  Recreated data (inverse FHT only)
        o  calculated coefficients (FHT only)


      resultTable coefficients are arranged as:
         Ŀ
        0           DC               
         Ĵ
        1 1st harmnonic (Real part)  
         Ĵ
        2 2nd harmnonic (Real part)  
         Ĵ
        3 3rd harmnonic (Real part)  
         
            .
         Ŀ
      N/2N/2th harmnonic (Real part) 
         
            .
         Ŀ
      N-4 4th harmnonic (Imag part)  
         Ĵ
      N-3 3rd harmnonic (Imag part)  
         Ĵ
      N-2 2nd harmnonic (Imag part)  
         Ĵ
      N-1 1st harmnonic (Imag part)  
         
*/

enum FHTFunction {calcFHT, inverseFHT};

void FHT (FHTFunction function,  unsigned int numberOfDataPoints,
          const double inputTable[], double resultTable[]);





/**********************************************************************/
/*                 PROCEDURE  SelectHarmonics                         */
/**********************************************************************/
/*
  Given an array created by FHT, above, select a range
  of harmonics to be included when performing the inverseFHT function.
  IN: firstHarmonic - lower Harmonic to include
      lastHarmonic  - upper harmonic in include
      coeff         - fourier coefficients from FHT, above.
 OUT: coeff         - array of only the chosen coefficients present.
*/
void SelectHarmonics(unsigned int firstHarmonic, unsigned int lastHarmonic,
                     unsigned int numberOfDataPoints,
                     double coeff[]);





/**********************************************************************/
/*                 PROCEDURE  CalculateHarmonic                       */
/**********************************************************************/
/*
Converts from rectangular FHT result to polar coordinates for the requested
harmonic.

IN:  coeffArray - Array from FHT.
     numberOfDataPoints - size of array
     harmonic - harmonic number to return
OUT: amplitude : peak-to-peak magnitude.
     angle    : angle of peak point in TENTHS of degrees
*/
void CalculateHarmonic(const double coeffArray[],
                       unsigned int numberOfDataPoints, 
                       unsigned int harmonic,
                       double & peakToPeak, double & angle);

#endif
